@@ -31,7 +31,7 @@ pub const fn default_axis() -> DefaultA {
 ///Expose a common Sorter trait so that we may have two version of the tree
 ///where one implementation actually does sort the tree, while the other one
 ///does nothing when sort() is called.
-pub trait Sorter: Copy + Clone + Send + Sync + Default {
+pub trait Sorter: Copy + Clone + Send + Sync {
     fn sort(&self, axis: impl Axis, bots: &mut [impl Aabb]);
 }
 
@@ -73,8 +73,8 @@ pub mod num_level {
             let (num_bots, num_per_node) = (num_bots as u64, num_per_node as u64);
             let a = num_bots / num_per_node;
             let a = log_2(a);
-            let k=(((a/2)*2)+1) as usize;
-            assert_eq!(k % 2,1,"k={:?}",k);
+            let k = (((a / 2) * 2) + 1) as usize;
+            assert_eq!(k % 2, 1, "k={:?}", k);
             k
         }
     }
@@ -112,26 +112,6 @@ pub fn bbox_mut<N, T>(rect: axgeom::Rect<N>, inner: &mut T) -> BBoxMut<N, T> {
     BBoxMut::new(rect, inner)
 }
 
-///
-/// Specify options for constructing the tree.
-///
-pub trait TreeBuild<T: Aabb, S: Sorter>: Sized {
-    fn sorter(&self) -> S;
-
-    fn num_level(&self, num_bots: usize) -> usize {
-        num_level::default(num_bots)
-    }
-    fn height_seq_fallback(&self) -> usize {
-        5
-    }
-}
-
-impl<T: Aabb, S: Sorter> TreeBuild<T, S> for S {
-    fn sorter(&self) -> S {
-        *self
-    }
-}
-
 ///Create a [`Tree`].
 ///
 /// # Examples
@@ -141,25 +121,22 @@ impl<T: Aabb, S: Sorter> TreeBuild<T, S> for S {
 /// let tree = broccoli_tree::new(&mut bots);
 ///
 ///```
-#[must_use]
 pub fn new<T: Aabb>(bots: &mut [T]) -> Tree<T> {
-    TreeInner::new(DefaultSorter, bots)
+    TreeBuilder::new(DefaultSorter, bots).build()
 }
 
-#[must_use]
 #[cfg(feature = "rayon")]
 pub fn new_par<T: Aabb>(bots: &mut [T]) -> Tree<T>
 where
     T: Send + Sync,
     T::Num: Send + Sync,
 {
-    TreeInner::new_par(DefaultSorter, bots)
+    TreeBuilder::new(DefaultSorter, bots).build_par()
 }
 
 ///
 /// Create a [`TreeOwned`]
 ///
-#[must_use]
 pub fn new_owned<C: Container>(cont: C) -> TreeOwned<C, DefaultSorter>
 where
     C::T: Aabb,
@@ -167,7 +144,6 @@ where
     TreeOwned::new(DefaultSorter, cont)
 }
 
-#[must_use]
 #[cfg(feature = "rayon")]
 pub fn new_owned_par<C: Container>(cont: C) -> TreeOwned<C, DefaultSorter>
 where
@@ -207,6 +183,7 @@ impl<T> Container for Box<[T]> {
 ///
 /// Owned version of [`Tree`]
 ///
+#[must_use]
 pub struct TreeOwned<C: Container, S>
 where
     C::T: Aabb,
@@ -220,12 +197,11 @@ impl<C: Container, S: Sorter> TreeOwned<C, S>
 where
     C::T: Aabb,
 {
-    #[must_use]
     pub fn new(sorter: S, mut bots: C) -> TreeOwned<C, S> {
         let j = bots.as_mut();
         let length = j.len();
 
-        let t = TreeInner::new(sorter, j);
+        let t = TreeBuilder::new(sorter, j).build();
         let data = t.into_node_data_tree();
         TreeOwned {
             inner: bots,
@@ -235,7 +211,6 @@ where
     }
 
     #[cfg(feature = "rayon")]
-    #[must_use]
     pub fn new_par(sorter: S, mut bots: C) -> TreeOwned<C, S>
     where
         C::T: Send + Sync,
@@ -244,7 +219,7 @@ where
         let j = bots.as_mut();
         let length = j.len();
 
-        let t = TreeInner::new_par(sorter, j);
+        let t = TreeBuilder::new(sorter, j).build_par();
         let data = t.into_node_data_tree();
         TreeOwned {
             inner: bots,
@@ -253,7 +228,6 @@ where
         }
     }
 
-    #[must_use]
     pub fn as_tree(&mut self) -> TreeInner<Node<C::T>, S> {
         let j = self.inner.as_mut();
         assert_eq!(j.len(), self.length);
@@ -287,48 +261,75 @@ where
 /// The main tree struct
 ///
 #[derive(Clone)]
+#[must_use]
 pub struct TreeInner<N, S> {
     total_num_elem: usize,
     nodes: Vec<N>,
     sorter: S,
 }
 
-///
-/// [`TreeInner`] type with default node and sorter.
-///
-pub type Tree<'a, T> = TreeInner<Node<'a, T>, DefaultSorter>;
+pub struct TreeBuilder<'a, T, S> {
+    pub bots: &'a mut [T],
+    pub sorter: S,
+    pub num_level: usize,
+    pub num_seq_fallback: usize,
+}
 
-impl<'a, T: Aabb + 'a, S: Sorter> TreeInner<Node<'a, T>, S> {
-    #[must_use]
-    pub fn new(tb: impl TreeBuild<T, S>, bots: &'a mut [T]) -> TreeInner<Node<'a, T>, S> {
+impl<'a, T: Aabb> TreeBuilder<'a, T, NoSorter> {
+    pub fn new_no_sort(bots: &'a mut [T]) -> Self {
+        Self::new(NoSorter, bots)
+    }
+}
+impl<'a, T: Aabb> TreeBuilder<'a, T, DefaultSorter> {
+    pub fn new_default(bots: &'a mut [T]) -> Self {
+        Self::new(DefaultSorter, bots)
+    }
+}
+impl<'a, T: Aabb, S: Sorter> TreeBuilder<'a, T, S> {
+    pub fn new(sorter: S, bots: &'a mut [T]) -> Self {
+        let num_bots = bots.len();
+        let num_seq_fallback = 2_400;
+        TreeBuilder {
+            bots,
+            sorter,
+            num_level: num_level::default(num_bots),
+            num_seq_fallback,
+        }
+    }
+    pub fn build(self) -> TreeInner<Node<'a, T>, S> {
+        let TreeBuilder {
+            bots,
+            sorter,
+            num_level,
+            ..
+        } = self;
         let total_num_elem = bots.len();
-        let num_level = tb.num_level(bots.len()); //num_level::default(bots.len());
         let mut buffer = Vec::with_capacity(num_level::num_nodes(num_level));
-        let vistr = TreeBuildVisitor::new(num_level, bots, tb.sorter());
+        let vistr = TreeBuildVisitor::new(num_level, bots, sorter);
         vistr.recurse_seq(&mut buffer);
         TreeInner {
             nodes: buffer,
-            sorter: tb.sorter(),
+            sorter,
             total_num_elem,
         }
     }
 
     #[cfg(feature = "rayon")]
-    #[must_use]
-    pub fn new_par(tb: impl TreeBuild<T, S>, bots: &'a mut [T]) -> TreeInner<Node<'a, T>, S>
+    pub fn build_par(self) -> TreeInner<Node<'a, T>, S>
     where
-        T: Send + Sync,
-        T::Num: Send + Sync,
+        T: Send,
+        T::Num: Send,
     {
         pub fn recurse_par<'a, T: Aabb, S: Sorter>(
             vistr: TreeBuildVisitor<'a, T, S>,
-            height_seq_fallback: usize,
+            num_seq_fallback: usize,
             buffer: &mut Vec<Node<'a, T>>,
         ) where
             T: Send,
             T::Num: Send,
         {
-            if vistr.get_height() <= height_seq_fallback {
+            if vistr.get_bots().len() <= num_seq_fallback {
+                //println!("switching at height={:?}",vistr.get_height());
                 vistr.recurse_seq(buffer);
             } else {
                 let NodeBuildResult { node, rest } = vistr.build_and_next();
@@ -337,11 +338,11 @@ impl<'a, T: Aabb + 'a, S: Sorter> TreeInner<Node<'a, T>, S> {
                     let (_, mut a) = rayon::join(
                         || {
                             buffer.push(node.finish());
-                            recurse_par(left, height_seq_fallback, buffer);
+                            recurse_par(left, num_seq_fallback, buffer);
                         },
                         || {
                             let mut f = vec![];
-                            recurse_par(right, height_seq_fallback, &mut f);
+                            recurse_par(right, num_seq_fallback, &mut f);
                             f
                         },
                     );
@@ -351,25 +352,37 @@ impl<'a, T: Aabb + 'a, S: Sorter> TreeInner<Node<'a, T>, S> {
             }
         }
 
+        let TreeBuilder {
+            bots,
+            sorter,
+            num_level,
+            num_seq_fallback,
+        } = self;
         let total_num_elem = bots.len();
-        let num_level = tb.num_level(bots.len()); //num_level::default(bots.len());
         let mut buffer = Vec::with_capacity(num_level::num_nodes(num_level));
-        let vistr = TreeBuildVisitor::new(num_level, bots, tb.sorter());
-        recurse_par(vistr, tb.height_seq_fallback(), &mut buffer);
+        let vistr = TreeBuildVisitor::new(num_level, bots, sorter);
+        recurse_par(vistr, num_seq_fallback, &mut buffer);
 
         TreeInner {
             nodes: buffer,
-            sorter: tb.sorter(),
+            sorter,
             total_num_elem,
         }
     }
+}
 
-    #[must_use]
+///
+/// [`TreeInner`] type with default node and sorter.
+///
+pub type Tree<'a, T> = TreeInner<Node<'a, T>, DefaultSorter>;
+
+impl<'a, T: Aabb + 'a, S: Sorter> TreeInner<Node<'a, T>, S> {
     pub fn into_node_data_tree(self) -> TreeInner<NodeData<T::Num>, S> {
         self.node_map(|x| NodeData {
             range: x.range.len(),
             cont: x.cont,
             div: x.div,
+            num_elem: x.num_elem,
         })
     }
 }
@@ -386,7 +399,6 @@ fn as_node_tree<N>(vec: &[N]) -> compt::dfs_order::CompleteTree<N, compt::dfs_or
 }
 
 impl<S, H> TreeInner<H, S> {
-    #[must_use]
     #[inline(always)]
     pub fn node_map<K>(self, func: impl FnMut(H) -> K) -> TreeInner<K, S> {
         let sorter = self.sorter;
@@ -434,21 +446,18 @@ impl<S, H> TreeInner<H, S> {
         AabbPin::from_mut(&mut self.nodes)
     }
 
-    #[must_use]
     #[inline(always)]
     pub fn vistr_mut(&mut self) -> VistrMutPin<H> {
         let tree = compt::dfs_order::CompleteTreeMut::from_preorder_mut(&mut self.nodes).unwrap();
         VistrMutPin::new(tree.vistr_mut())
     }
 
-    #[must_use]
     #[inline(always)]
     pub fn vistr_mut_raw(&mut self) -> compt::dfs_order::VistrMut<H, compt::dfs_order::PreOrder> {
         let tree = compt::dfs_order::CompleteTreeMut::from_preorder_mut(&mut self.nodes).unwrap();
         tree.vistr_mut()
     }
 
-    #[must_use]
     #[inline(always)]
     pub fn vistr(&self) -> Vistr<H> {
         let tree = as_node_tree(&self.nodes);
@@ -467,7 +476,6 @@ impl<S, H> TreeInner<H, S> {
 }
 
 impl<N: Num, S: Sorter> TreeInner<NodeData<N>, S> {
-    #[must_use]
     pub fn into_tree<T: Aabb<Num = N>>(self, bots: AabbPin<&mut [T]>) -> TreeInner<Node<T>, S> {
         assert_eq!(bots.len(), self.total_num_elem);
         let mut last = Some(bots);
@@ -478,6 +486,7 @@ impl<N: Num, S: Sorter> TreeInner<NodeData<N>, S> {
                 range,
                 cont: x.cont,
                 div: x.div,
+                num_elem: x.num_elem,
             }
         });
         assert!(last.unwrap().is_empty());
